#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
# All rights reserved.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
''' Provides a command line application for Bokeh.

The following subcommands are available:

html
    Create standalone HTML files for one or more applications

info
    print information about Bokeh and Bokeh server configuration

json
    Create JSON files for one or more applications

png
    Create standalone PNG files for one or more applications

sampledata
    Download the bokeh sample data sets

secret
    Create a Bokeh secret key for use with Bokeh server

serve
    Run a Bokeh server hosting one or more applications

static
    Serve bokehjs' static assets (JavaScript, CSS, images, fonts, etc.)

svg
    Create standalone SVG files for one or more applications

'''
