import "mixins.css";
export declare const bk_active = "bk-active";
export declare const bk_inline = "bk-inline";
export declare const bk_left = "bk-left";
export declare const bk_right = "bk-right";
export declare const bk_above = "bk-above";
export declare const bk_below = "bk-below";
export declare const bk_up = "bk-up";
export declare const bk_down = "bk-down";
import { Side } from "../core/enums";
export declare function bk_side(side: Side): string;
